@@ -36,6 +36,7 @@
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <eel/eel-string.h>
 
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagesmanager.h>
@@ -58,6 +59,9 @@
 #include "recent-files/egg-recent-view-gtk.h"
 #include "recent-files/egg-recent-model.h"
 #include "gedit-languages-manager.h"
+#include "dialogs/gedit-close-confirmation-dialog.h"
+
+#include <eel/eel-alert-dialog.h>
 
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-control.h>
@@ -69,7 +73,7 @@
 
 struct _GeditMDIPrivate
 {
-	gint dummy;
+	GeditState state;
 };
 
 typedef struct _GeditWindowPrefs	GeditWindowPrefs;
@@ -99,7 +103,9 @@ static void gedit_mdi_set_app_statusbar_style 	(BonoboWindow *win);
 static gint gedit_mdi_add_child_handler (BonoboMDI *mdi, BonoboMDIChild *child);
 static gint gedit_mdi_add_view_handler (BonoboMDI *mdi, GtkWidget *view);
 static gint gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child);
-static gint gedit_mdi_remove_view_handler (BonoboMDI *mdi, GtkWidget *view);
+
+static gboolean gedit_mdi_remove_view_handler  (BonoboMDI *mdi, GtkWidget *view);
+static gboolean gedit_mdi_remove_views_handler (BonoboMDI *mdi, BonoboWindow *window);
 
 static void gedit_mdi_view_changed_handler (BonoboMDI *mdi, GtkWidget *old_view);
 static void gedit_mdi_child_changed_handler (BonoboMDI *mdi, BonoboMDIChild *old_child);
@@ -127,7 +133,14 @@ static void		 gedit_window_prefs_attach_to_window 	(GeditWindowPrefs *prefs,
 static GeditWindowPrefs	*gedit_window_prefs_get_from_window 	(BonoboWindow     *win);
 static void		 gedit_window_prefs_save 		(GeditWindowPrefs *prefs);
 
+enum 
+{
+	STATE_CHANGED = 0,
+	LAST_SIGNAL
+};
+
 static BonoboMDIClass *parent_class = NULL;
+static guint mdi_signals [LAST_SIGNAL] = { 0 };
 
 enum
 {
@@ -179,6 +192,17 @@ gedit_mdi_class_init (GeditMDIClass *klass)
   	parent_class = g_type_class_peek_parent (klass);
 
   	object_class->finalize = gedit_mdi_finalize;
+
+	mdi_signals[STATE_CHANGED] = 
+		g_signal_new ("state_changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GeditMDIClass, state_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__INT,
+			      G_TYPE_NONE, 
+			      1, 
+			      G_TYPE_INT);
 }
 
 static void
@@ -266,7 +290,7 @@ tooltip_func (GtkTooltips   *tooltips,
 	g_free (tip);
 }
 
-static void
+static GtkWidget *
 gedit_mdi_add_open_button (GeditMDI *mdi, BonoboUIComponent *ui_component,
 			 const gchar *path, const gchar *tooltip)
 {
@@ -313,6 +337,8 @@ gedit_mdi_add_open_button (GeditMDI *mdi, BonoboUIComponent *ui_component,
 
 	bonobo_ui_component_widget_set (ui_component, path, GTK_WIDGET (button),
 					NULL);
+
+	return button;
 }
 
 static void 
@@ -328,6 +354,8 @@ gedit_mdi_init (GeditMDI  *mdi)
 	
 	mdi->priv = g_new0 (GeditMDIPrivate, 1);
 
+	mdi->priv->state = GEDIT_STATE_NORMAL;
+
 	bonobo_mdi_set_ui_template_file (BONOBO_MDI (mdi), GEDIT_UI_DIR "gedit-ui.xml", gedit_verbs);
 	
 	bonobo_mdi_set_child_list_path (BONOBO_MDI (mdi), "/menu/Documents/OpenDocuments/");
@@ -345,6 +373,8 @@ gedit_mdi_init (GeditMDI  *mdi)
 			  G_CALLBACK (gedit_mdi_remove_child_handler), NULL);
 	g_signal_connect (G_OBJECT (mdi), "remove_view",
 			  G_CALLBACK (gedit_mdi_remove_view_handler), NULL);
+	g_signal_connect (G_OBJECT (mdi), "remove_views",
+			  G_CALLBACK (gedit_mdi_remove_views_handler), NULL);
 
 	g_signal_connect (G_OBJECT (mdi), "child_changed",
 			  G_CALLBACK (gedit_mdi_child_changed_handler), NULL);
@@ -481,11 +511,13 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	bonobo_object_unref (BONOBO_OBJECT (control));
 
 	g_object_set_data (G_OBJECT (win), "OverwriteMode", widget);
-
+	
 	/* Add custom Open button to toolbar */
-	gedit_mdi_add_open_button (GEDIT_MDI (mdi),
-				   ui_component, "/Toolbar/FileOpenMenu",
-				   _("Open a recently used file"));
+	widget = gedit_mdi_add_open_button (GEDIT_MDI (mdi),
+					    ui_component, 
+					    "/Toolbar/FileOpenMenu",
+					    _("Open a recently used file"));
+	g_object_set_data (G_OBJECT (win), "recent-menu-button", widget);
 
 	prefs = gedit_window_prefs_new ();
 	gedit_window_prefs_attach_to_window (prefs, win);
@@ -700,7 +732,7 @@ gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context
 				      guint info, guint time)
 {
 	GList *list = NULL;
-	GList *file_list = NULL;
+	GSList *file_list = NULL;
 	GList *p = NULL;
 	
 	gedit_debug (DEBUG_MDI, "");
@@ -713,24 +745,24 @@ gedit_mdi_drag_data_received_handler (GtkWidget *widget, GdkDragContext *context
 
 	while (p != NULL)
 	{
-		file_list = g_list_append (file_list, 
+		file_list = g_slist_prepend (file_list, 
 				gnome_vfs_uri_to_string ((const GnomeVFSURI*)(p->data), 
 				GNOME_VFS_URI_HIDE_NONE));
-		p = p->next;
+
+		p = g_list_next (p);
 	}
 	
 	gnome_vfs_uri_list_free (list);
 
-	gedit_file_open_uri_list (file_list, 0, FALSE);	
-	
+	file_list = g_slist_reverse (file_list);
+
 	if (file_list == NULL)
 		return;
 
-	for (p = file_list; p != NULL; p = p->next) {
-		g_free (p->data);
-	}
+	gedit_file_open_uri_list (file_list, 0, FALSE);	
 	
-	g_list_free (file_list);
+	g_slist_foreach (file_list, (GFunc)g_free, NULL);	
+	g_slist_free (file_list);
 }
 
 static void
@@ -941,164 +973,304 @@ gedit_mdi_add_view_handler (BonoboMDI *mdi, GtkWidget *view)
 	return TRUE;
 }
 
+
 static gint 
 gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 {
 	GeditDocument* doc;
 	gboolean close = TRUE;
-	gchar *raw_uri;
-	gboolean deleted = FALSE;
 	
 	gedit_debug (DEBUG_MDI, "");
 
 	g_return_val_if_fail (child != NULL, FALSE);
 	g_return_val_if_fail (GEDIT_MDI_CHILD (child)->document != NULL, FALSE);
 
+	if (gedit_mdi_child_get_closing (GEDIT_MDI_CHILD (child)))
+	{
+		return TRUE;
+	}
+
 	doc = GEDIT_MDI_CHILD (child)->document;
 
-	raw_uri = gedit_document_get_raw_uri (doc); 
-	if (raw_uri != NULL)
+	if (gedit_document_get_modified (doc) || gedit_document_get_deleted (doc))
 	{
-		if (gedit_document_is_readonly (doc))
-			deleted = FALSE;
-		else
-			deleted = !gedit_utils_uri_exists (raw_uri);
-	}
-	g_free (raw_uri);
-				
-	if (gedit_document_get_modified (doc) || deleted)
-	{
-		GtkWidget *msgbox, *w;
-		gchar *fname = NULL, *msg = NULL;
-		gint ret;
-		gboolean exiting;
-
-		w = GTK_WIDGET (g_list_nth_data (bonobo_mdi_child_get_views (child), 0));
+		GtkWidget *view;
+		GtkWindow *window;
+		gboolean   save;
+		GtkWidget *dlg;
+		
+		window = NULL;
+		view = GTK_WIDGET (g_list_nth_data (bonobo_mdi_child_get_views (child), 0));
 			
-		if(w != NULL)
+		if (view != NULL)
 		{
-			GtkWindow *window;
-
-			window = GTK_WINDOW (bonobo_mdi_get_window_from_view (w));
+			window = GTK_WINDOW (bonobo_mdi_get_window_from_view (view));
 			gtk_window_present (window);
 			
-			bonobo_mdi_set_active_view (mdi, w);
+			bonobo_mdi_set_active_view (mdi, view);
 		}
 
-		fname = gedit_document_get_short_name (doc);
+		dlg = gedit_close_confirmation_dialog_new_single (window, doc);
+		
+		save = gedit_close_confirmation_dialog_run (GEDIT_CLOSE_CONFIRMATION_DIALOG (dlg));
+		
+		gtk_widget_hide (dlg);
 
-		msgbox = gtk_message_dialog_new (GTK_WINDOW (bonobo_mdi_get_active_window (mdi)),
-				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				_("Do you want to save the changes you made to the document \"%s\"? \n\n"
-				  "Your changes will be lost if you don't save them."),
-				fname);
-
-		gedit_dialog_add_button (GTK_DIALOG (msgbox),
-				_("Do_n't save"), GTK_STOCK_NO,
-				GTK_RESPONSE_NO);
-
-		if (gedit_close_x_button_pressed)
-			exiting = FALSE;
-		else if (gedit_exit_button_pressed)
-			exiting = TRUE;
-		else
+		if (save)
 		{
-			/* Delete event generated */
-			if (g_list_length (bonobo_mdi_get_windows (BONOBO_MDI (gedit_mdi))) == 1)
-				exiting = TRUE;
-			else
-				exiting = FALSE;
-		}
+			GSList *sel_docs;
 
-#if 0		
-		if (exiting)
-			gedit_dialog_add_button (GTK_DIALOG (msgbox),
-					_("_Don't quit"), GTK_STOCK_CANCEL,
-                	             	GTK_RESPONSE_CANCEL);
-		else
-			gedit_dialog_add_button (GTK_DIALOG (msgbox),
-					_("_Don't close"), GTK_STOCK_CANCEL,
-                	             	GTK_RESPONSE_CANCEL);
-#endif
-		
-		gtk_dialog_add_button (GTK_DIALOG (msgbox), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+			sel_docs = gedit_close_confirmation_dialog_get_selected_documents (
+						GEDIT_CLOSE_CONFIRMATION_DIALOG (dlg));
 
-		gtk_dialog_add_button (GTK_DIALOG (msgbox),
-			       	GTK_STOCK_SAVE,
-				GTK_RESPONSE_YES);
-
-		gtk_dialog_set_default_response	(GTK_DIALOG (msgbox), GTK_RESPONSE_YES);
-
-		gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
-
-		gtk_widget_show (msgbox);
-		gtk_window_present (GTK_WINDOW (msgbox));
-
-		ret = gtk_dialog_run (GTK_DIALOG (msgbox));
-		
-		gtk_widget_destroy (msgbox);
-
-		g_free (fname);
-		g_free (msg);
-		
-		switch (ret)
-		{
-			case GTK_RESPONSE_YES:
-				close = gedit_file_save (GEDIT_MDI_CHILD (child), TRUE);
-				break;
-			case GTK_RESPONSE_NO:
+			if (sel_docs == NULL)
+			{
 				close = TRUE;
-				break;
-			default:
-				close = FALSE;
+			}
+			else
+			{
+				close = gedit_file_save (GEDIT_MDI_CHILD (child), TRUE);
+
+				g_slist_free (sel_docs);
+			}
 		}
+		else
+		{
+			close = FALSE;
+		}
+
+		gtk_widget_destroy (dlg);
 
 		gedit_debug (DEBUG_MDI, "CLOSE: %s", close ? "TRUE" : "FALSE");
 	}
-	
-	/* FIXME: there is a bug if you "close all" >1 docs, don't save the document
-	 * and then don't close the last one.
-	 */
-	/* Disable to avoid the bug */
-	/*
-	if (close)
-	{
-		g_signal_handlers_disconnect_by_func (child, 
-						      G_CALLBACK (gedit_mdi_child_state_changed_handler),
-						      NULL);
-		g_signal_handlers_disconnect_by_func (GTK_OBJECT (child), 
-						      G_CALLBACK (gedit_mdi_child_undo_redo_state_changed_handler),
-						      NULL);
-		g_signal_handlers_disconnect_by_func (child, 
-						      G_CALLBACK (gedit_mdi_child_find_state_changed_handler),
-						      NULL);
-	}
-	*/
-	
+		
 	return close;
 }
 
-static gint 
-gedit_mdi_remove_view_handler (BonoboMDI *mdi,  GtkWidget *view)
+static gboolean
+gedit_mdi_remove_view_handler (BonoboMDI *mdi, GtkWidget *view)
 {
 	gedit_debug (DEBUG_MDI, "");
 
 	return TRUE;
 }
 
-void 
-gedit_mdi_set_active_window_title (BonoboMDI *mdi)
+static gboolean
+gedit_mdi_can_remove_views (GList *views, BonoboWindow *window)
 {
-	BonoboMDIChild* active_child = NULL;
-	GeditDocument* doc = NULL;
-	gchar* docname = NULL;
-	gchar* title = NULL;
+	GList *l;
+	GSList *unsaved_docs;
+	GtkWidget *dlg;
+	gboolean close;
 	
 	gedit_debug (DEBUG_MDI, "");
 
+	if (window == NULL)
+	{
+		window = gedit_get_active_window ();
+	}
+
+	unsaved_docs = NULL;
+
+	l = views;
+	while (l != NULL)
+	{
+		GeditView *view;
+		GeditDocument *doc;
+		gchar *raw_uri;
+		gboolean deleted;
+		
+		view = GEDIT_VIEW (l->data);
+		
+		doc = gedit_view_get_document (view);
+		g_return_val_if_fail (doc != NULL, FALSE);
+		
+		deleted = FALSE;
+		
+		raw_uri = gedit_document_get_raw_uri (doc); 
+		if (raw_uri != NULL)
+		{
+			if (gedit_document_is_readonly (doc))
+				deleted = FALSE;
+			else
+				deleted = !gedit_utils_uri_exists (raw_uri);
+		}
+		g_free (raw_uri);
+
+		if (gedit_document_get_modified (doc) || deleted)
+		{
+			unsaved_docs = g_slist_prepend (unsaved_docs, doc);
+		}
+
+		l = g_list_next (l);
+	}
 	
+	if (unsaved_docs == NULL)
+	{
+		l = views;
+		while (l != NULL)
+		{
+			GeditView *view;
+			BonoboMDIChild *child;
+			
+			view = GEDIT_VIEW (l->data);
+
+			child =	bonobo_mdi_get_child_from_view (GTK_WIDGET (view));
+			g_return_val_if_fail (child != NULL, FALSE);
+			
+			gedit_mdi_child_set_closing (GEDIT_MDI_CHILD (child),
+						     TRUE);
+						     
+			l = g_list_next (l);
+		}
+		
+		return TRUE;
+	}
+
+	unsaved_docs = g_slist_reverse (unsaved_docs);
+
+	if (g_slist_length (unsaved_docs) == 1)
+	{
+		GtkWidget *view;
+		GeditMDIChild *child;
+
+		child = gedit_mdi_child_get_from_document (GEDIT_DOCUMENT (unsaved_docs->data));
+
+		view = GTK_WIDGET (g_list_nth_data (
+					bonobo_mdi_child_get_views (BONOBO_MDI_CHILD (child)), 0));
+			
+		if (view != NULL)
+		{
+			window = bonobo_mdi_get_window_from_view (view);
+			gtk_window_present (GTK_WINDOW (window));
+			
+			bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
+		}
+	}	
+	
+	dlg = gedit_close_confirmation_dialog_new (GTK_WINDOW (window), unsaved_docs);
+
+	close = gedit_close_confirmation_dialog_run (GEDIT_CLOSE_CONFIRMATION_DIALOG (dlg));
+		
+	gtk_widget_hide (dlg);
+
+	if (close)
+	{
+		GSList *sel_docs;
+
+		sel_docs = gedit_close_confirmation_dialog_get_selected_documents
+				(GEDIT_CLOSE_CONFIRMATION_DIALOG (dlg));
+
+		if (sel_docs != NULL)
+		{
+			GSList *sl;
+
+			sl = sel_docs;
+			while (sl != NULL)
+			{
+				GeditMDIChild *child;
+				GeditDocument *doc;
+
+				doc = GEDIT_DOCUMENT (sl->data);
+				child = gedit_mdi_child_get_from_document (doc);
+				g_return_val_if_fail (child != NULL, FALSE);
+				
+				close &= gedit_file_save (child, FALSE);
+				
+				sl = g_slist_next (sl);
+			}
+
+			g_slist_free (sel_docs);
+		}
+	}
+
+	gtk_widget_destroy (dlg);
+
+	if (close)
+	{
+		l = views;
+		while (l != NULL)
+		{
+			GeditView *view;
+			BonoboMDIChild *child;
+			
+			view = GEDIT_VIEW (l->data);
+
+			child =	bonobo_mdi_get_child_from_view (GTK_WIDGET (view));
+			g_return_val_if_fail (child != NULL, FALSE);
+			
+			gedit_mdi_child_set_closing (GEDIT_MDI_CHILD (child),
+						     TRUE);
+						     
+			l = g_list_next (l);
+		}
+	
+	}
+
+	g_slist_free (unsaved_docs);
+
+	return close;
+}
+
+static gboolean
+gedit_mdi_remove_views_handler (BonoboMDI *mdi, BonoboWindow *window)
+{
+	GList *views;
+	gboolean ret;
+	
+	gedit_debug (DEBUG_MDI, "");
+
+	if (gedit_mdi_get_state (GEDIT_MDI (mdi)) != GEDIT_STATE_NORMAL)
+		return FALSE;
+
+	views = bonobo_mdi_get_views_from_window (mdi, window);
+
+	ret = gedit_mdi_can_remove_views (views, window);
+
+	g_list_free (views);
+
+	return ret;
+}
+
+gboolean 
+gedit_mdi_remove_all (GeditMDI *mdi)
+{
+	GList *views;
+	gboolean ret;
+	
+	gedit_debug (DEBUG_MDI, "");
+
+	g_return_val_if_fail (GEDIT_IS_MDI (mdi), FALSE);
+
+	views = bonobo_mdi_get_views (BONOBO_MDI (mdi));
+
+	ret = gedit_mdi_can_remove_views (views, NULL);
+
+	g_list_free (views);
+
+	if (ret)
+	{
+		ret = bonobo_mdi_remove_all (BONOBO_MDI (mdi),
+					     TRUE);
+	}
+
+	return ret;
+}
+	
+#define MAX_URI_IN_TITLE_LENGTH 75
+
+void 
+gedit_mdi_set_active_window_title (BonoboMDI *mdi)
+{
+	BonoboMDIChild *active_child = NULL;
+	GeditDocument *doc = NULL;
+	gchar *docname = NULL;
+	gchar *title = NULL;
+	gchar *uri;
+	GtkWidget *active_window;
+	
+	gedit_debug (DEBUG_MDI, "");
+
 	active_child = bonobo_mdi_get_active_child (mdi);
 	if (active_child == NULL)
 		return;
@@ -1107,8 +1279,12 @@ gedit_mdi_set_active_window_title (BonoboMDI *mdi)
 	g_return_if_fail (doc != NULL);
 	
 	/* Set active window title */
-	docname = gedit_document_get_uri (doc);
-	g_return_if_fail (docname != NULL);
+	uri = gedit_document_get_uri (doc);
+	g_return_if_fail (uri != NULL);
+
+	/* Truncate the URI so it doesn't get insanely wide. */
+	docname = eel_str_middle_truncate (uri, MAX_URI_IN_TITLE_LENGTH);
+	g_free (uri);
 
 	if (gedit_document_get_modified (doc))
 	{
@@ -1127,7 +1303,19 @@ gedit_mdi_set_active_window_title (BonoboMDI *mdi)
 
 	}
 
-	gtk_window_set_title (GTK_WINDOW (bonobo_mdi_get_active_window (mdi)), title);
+	active_window = GTK_WIDGET (gedit_get_active_window ());
+
+	if (GTK_WIDGET_REALIZED (active_window))
+	{	
+		gchar *short_name;
+		
+		short_name = gedit_document_get_short_name (doc);
+		
+		gdk_window_set_icon_name (active_window->window, short_name);
+		g_free (short_name);
+	}
+
+	gtk_window_set_title (GTK_WINDOW (active_window), title);
 	
 	g_free (docname);
 	g_free (title);
@@ -1229,6 +1417,12 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	
 	gedit_plugins_engine_update_plugins_ui (active_window, FALSE);
 	
+	gedit_menus_set_verb_list_sensitive (ui_component, 
+				gedit_menus_all_sensible_verbs, TRUE);
+
+	bonobo_ui_component_set_prop (
+			ui_component, "/menu/View/HighlightMode", "sensitive", "1", NULL);
+	
 	if (active_child == NULL)
 	{
 		gedit_menus_set_verb_list_sensitive (ui_component, 
@@ -1239,15 +1433,6 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 
 		goto end;
 	}
-	else
-	{
-		gedit_menus_set_verb_list_sensitive (ui_component, 
-				gedit_menus_all_sensible_verbs, TRUE);
-
-		bonobo_ui_component_set_prop (
-			ui_component, "/menu/View/HighlightMode", "sensitive", "1", NULL);
-
-	}
 
 	gedit_menus_set_verb_sensitive (ui_component, "/commands/DocumentsMoveToNewWindow",
 				(bonobo_mdi_n_children_for_window (active_window) > 1) ? TRUE : FALSE);
@@ -1278,7 +1463,6 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	{
 		gedit_menus_set_verb_list_sensitive (ui_component, 
 				gedit_menus_not_modified_doc_sensible_verbs, FALSE);
-		goto end;
 	}
 
 	if (gedit_document_is_untitled (doc))
@@ -1288,6 +1472,33 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	}
 
 end:
+
+	switch (gedit_mdi_get_state (GEDIT_MDI (mdi)))
+	{
+		case GEDIT_STATE_LOADING:
+		case GEDIT_STATE_SAVING:
+		case GEDIT_STATE_REVERTING:
+			gedit_menus_set_verb_list_sensitive (ui_component,
+				gedit_menus_loading_sensible_verbs, FALSE);
+			
+			bonobo_ui_component_set_prop (ui_component, 
+					"/menu/View/HighlightMode", "sensitive", "0", NULL);
+
+			break;
+
+		case GEDIT_STATE_PRINTING:
+			gedit_menus_set_verb_list_sensitive (ui_component,
+				gedit_menus_printing_sensible_verbs, FALSE);
+
+			bonobo_ui_component_set_prop (ui_component, 
+					"/menu/View/HighlightMode", "sensitive", "0", NULL);
+
+			break;
+		default:
+			/* Do nothing */
+			break;
+	}
+
 	bonobo_ui_component_thaw (ui_component, NULL);
 }
 
@@ -1847,3 +2058,126 @@ void gedit_mdi_update_languages_menu (BonoboMDI *mdi)
 	g_free (verb_name);
 }
 
+static void
+update_ui_according_to_state (GeditMDI *mdi)
+{
+	GList *views;
+	GdkCursor *cursor;
+	GList *windows;
+
+	gedit_debug (DEBUG_MDI, "");
+	
+	/* Upate menus and toolbars */
+	gedit_mdi_set_active_window_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
+
+	/* Update views editability */
+	views = bonobo_mdi_get_views (BONOBO_MDI (mdi));
+	while (views != NULL)
+	{
+		GeditView *view;
+		GeditDocument *doc;
+
+		view = GEDIT_VIEW (views->data);
+		doc = gedit_view_get_document (view);
+		
+		switch (gedit_mdi_get_state (mdi))
+		{
+			case GEDIT_STATE_NORMAL:
+				if (!gedit_document_is_readonly (doc))
+				{
+					gedit_view_set_editable (view, TRUE);
+				}
+
+				break;
+
+			case GEDIT_STATE_LOADING:
+			case GEDIT_STATE_PRINTING:
+			case GEDIT_STATE_SAVING:
+			case GEDIT_STATE_REVERTING:
+				gedit_view_set_editable (view, FALSE);
+				break;
+		}
+
+		views = g_list_next (views);
+	}
+
+	g_list_free (views);
+
+	/* Change cursor */
+	if (gedit_mdi_get_state (mdi) == GEDIT_STATE_NORMAL)
+	{
+		cursor = NULL;
+	}
+	else
+	{
+		cursor =  gdk_cursor_new_for_display (
+				gtk_widget_get_display (GTK_WIDGET (gedit_get_active_window ())),
+				GDK_WATCH);
+	}
+
+	windows = bonobo_mdi_get_windows (BONOBO_MDI (mdi));
+	while (windows != NULL)
+	{
+		GtkWidget *win;
+		gpointer recent_menu_button;
+
+		g_return_if_fail (windows->data != NULL);
+		
+		win = GTK_WIDGET (windows->data);
+		
+		gdk_window_set_cursor (win->window, cursor);
+
+		recent_menu_button = g_object_get_data (G_OBJECT (win), "recent-menu-button");
+
+		if (recent_menu_button != NULL)
+		{
+			gtk_widget_set_sensitive (GTK_WIDGET (recent_menu_button),
+						  gedit_mdi_get_state (mdi) == GEDIT_STATE_NORMAL);
+		}
+
+		windows = g_list_next (windows);
+	}
+
+	if (cursor != NULL)
+		gdk_cursor_unref (cursor);
+
+
+	/* TODO: Update/Add state icon - Paolo (Jan 13, 2004) */
+}
+
+GeditState
+gedit_mdi_get_state (GeditMDI *mdi)
+{
+	gedit_debug (DEBUG_MDI, "");
+
+	g_return_val_if_fail (GEDIT_IS_MDI (mdi), GEDIT_STATE_NORMAL);
+
+	return mdi->priv->state;
+}
+
+void
+gedit_mdi_set_state (GeditMDI   *mdi,
+		     GeditState  state)
+{
+	gedit_debug (DEBUG_MDI, "");
+
+	g_return_if_fail (GEDIT_IS_MDI (mdi));
+	g_return_if_fail ((state == GEDIT_STATE_NORMAL) ||
+			  (state == GEDIT_STATE_LOADING) ||
+			  (state == GEDIT_STATE_SAVING) ||
+			  (state == GEDIT_STATE_PRINTING) ||
+			  (state == GEDIT_STATE_REVERTING));
+
+	if (state != mdi->priv->state)
+	{
+		mdi->priv->state = state;
+		
+		update_ui_according_to_state (mdi);
+
+		g_signal_emit (G_OBJECT (mdi), 
+			       mdi_signals [STATE_CHANGED],
+			       0,
+			       state);
+	}
+}
+
